# -*- coding: utf-8 -*-

"""PyKEEN is a package for training and evaluating knowledge graph embedding.

Installation
------------
Get the Latest
~~~~~~~~~~~~~~~
Download the most recent code from ... with:

For Developers
~~~~~~~~~~~~~~
Clone the repository from `GitHub <https://github.com/SmartDataAnalytics/pykeen.git>'
"""

from .run import run

__version__ = '0.0.11'

__title__ = 'pykeen'
__description__ = "A package for training and evaluating knowledge graph embeddings"
__url__ = 'https://github.com/SmartDataAnalytics/PyKEEN.git'
__author__ = 'Mehdi Ali'
__email__ = 'mehdi.ali@cs.uni-bonn.de'
__license__ = 'MIT License'
__copyright__ = 'Copyright (c) 2018 Mehdi Ali'
