# -*- coding: utf-8 -*-

"""Implementations of various knowledge graph embedding models."""

from pykeen.kg_embeddings_model.conv_e import ConvE
from pykeen.kg_embeddings_model.distmult import DistMult
from pykeen.kg_embeddings_model.ermlp import ERMLP
from pykeen.kg_embeddings_model.rescal import RESCAL
from pykeen.kg_embeddings_model.structured_embedding import StructuredEmbedding
from pykeen.kg_embeddings_model.trans_d import TransD
from pykeen.kg_embeddings_model.trans_e import TransE
from pykeen.kg_embeddings_model.trans_h import TransH
from pykeen.kg_embeddings_model.trans_r import TransR
from pykeen.kg_embeddings_model.unstructured_model import UnstructuredModel
