# -*- coding: utf-8 -*-

"""Utilities for the command line interface."""

from pykeen.utilities.cli_utils.distmult_cli import configure_distmult_training_pipeline
from pykeen.utilities.cli_utils.ermlp_cli import configure_ermlp_training_pipeline
from pykeen.utilities.cli_utils.rescal_cli import configure_rescal_training_pipeline
from pykeen.utilities.cli_utils.structured_embedding_cli import configure_se_training_pipeline
from pykeen.utilities.cli_utils.trans_d_cli import configure_trans_d_training_pipeline
from pykeen.utilities.cli_utils.trans_e_cli import configure_trans_e_training_pipeline
from pykeen.utilities.cli_utils.trans_h_cli import configure_trans_h_training_pipeline
from pykeen.utilities.cli_utils.trans_r_cli import configure_trans_r_training_pipeline
from pykeen.utilities.cli_utils.unstructured_model_cli import configure_um_training_pipeline
